/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.cocoon;

import java.io.IOException;
import java.io.Serializable;

import org.apache.excalibur.source.SourceValidity;
import org.apache.excalibur.store.Store;

import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.thread.ThreadSafe;

import net.sourceforge.chaperon.parser.ParserTable;

/**
 * This class represent a store to cache the parsertables.
 *
 * @author Stephan Michels
 * @version CVS $Id: ParserTableStore.java,v 1.2 2002/05/31 13:59:59 benedikta Exp $
 */
public class ParserTableStore extends AbstractLogEnabled implements Component, ThreadSafe, Composable, Disposable
{
  final static String ROLE = "net.sourceforge.chaperon.cocoon.ParserTableStore";

  private ComponentManager _manager = null;
  private Store            _store   = null;

  public void compose (ComponentManager manager) throws ComponentException 
  {
    _manager = manager;
    //_store = (Store)_manager.lookup(Store.PERSISTENT_STORE);
    _store = (Store)_manager.lookup(Store.TRANSIENT_STORE);
  }

  public void dispose() 
  {
    _manager.release(_store);
    _store = null;
  }

  public void store(String systemid, ParserTable parsertable, SourceValidity validity)
  {
    try
    {
      _store.store(systemid, new ParserTableEntry(parsertable, validity));
    } catch (IOException ioe)
    {
      throw new IllegalStateException(ioe.getMessage());
    }
  }

  public boolean hasParserTable(String systemid)
  {
    return _store.get(systemid)!=null;
  }

  public ParserTable getParserTable(String systemid)
  {
    ParserTableEntry entry = (ParserTableEntry)_store.get(systemid);
    if (entry!=null)
      return entry.getParserTable();
    return null;
  }

  public SourceValidity getParserTableValidity(String systemid)
  {
    ParserTableEntry entry = (ParserTableEntry)_store.get(systemid);
    if (entry!=null)
      return entry.getValidity();
    return null;
  }

  public void remove(String systemid)
  {
    _store.remove(systemid);
  }
}
